@@ -44,9 +44,10 @@ QoreStringNode* Jvm::createVM() {
     vm_args.ignoreUnrecognized = false;
     vm_args.nOptions = 0;
 
-    size_t num_options = 1;
+    size_t num_options = 2;
     bool disable_jit = false;
-    QoreString min_heap, max_heap;
+    QoreString min_heap, max_heap, debug_cmd;
+    std::vector<std::string> strvec;
     // check QORE_JNI_DISABLE_JIT environment variable
     {
         QoreString val;
@@ -57,13 +58,43 @@ QoreStringNode* Jvm::createVM() {
                 //printd(5, "jni module: disabling JIT\n");
             }
         }
-        if (!SystemEnvironment::get("QORE_JNI_MIN_HEAP_SIZE", min_heap)) {
-            ++num_options;
-            min_heap.prepend("-Xms");
+    }
+    if (!SystemEnvironment::get("QORE_JNI_MIN_HEAP_SIZE", min_heap)) {
+        ++num_options;
+        min_heap.prepend("-Xms");
+    }
+    if (!SystemEnvironment::get("QORE_JNI_MAX_HEAP_SIZE", max_heap)) {
+        ++num_options;
+        max_heap.prepend("-Xmx");
+    }
+    {
+        QoreString val;
+        if (!SystemEnvironment::get("QORE_JNI_DEBUG", val)) {
+            int port = atoi(val.c_str());
+            if (port > 0) {
+                // enable Java debugger
+                ++num_options;
+                debug_cmd.sprintf("-agentlib:jdwp=transport=dt_socket,server=y,suspend=n,address=%d", port);
+            }
         }
-        if (!SystemEnvironment::get("QORE_JNI_MAX_HEAP_SIZE", max_heap)) {
-            ++num_options;
-            max_heap.prepend("-Xmx");
+    }
+    {
+        QoreString val;
+        if (!SystemEnvironment::get("QORE_JNI_JVM_ARGS", val)) {
+            ssize_t pos = 0;
+            while (true) {
+                ssize_t i = val.find(' ', pos);
+                if (i == -1) {
+                    std::string opt(val.c_str() + pos);
+                    strvec.push_back(opt);
+                    break;
+                }
+                // add option
+                std::string opt(val.c_str() + pos, i - pos);
+                strvec.push_back(opt);
+            }
+            // add final option
+            num_options += strvec.size();
         }
     }
 #ifdef QORE_JNI_SUPPORT_CLASSPATH
@@ -76,10 +107,17 @@ QoreStringNode* Jvm::createVM() {
     JavaVMOption options[num_options];
     // "reduced signals"
     options[vm_args.nOptions++].optionString = (char*)"-Xrs";
+    // thread stack size
+    QoreStringMaker thread_stack_size("-Xss%ldk", q_thread_get_stack_size() / 1024);
+    options[vm_args.nOptions++].optionString = (char*)thread_stack_size.c_str();
     if (disable_jit) {
         // disable JIT
         options[vm_args.nOptions++].optionString = (char*)"-Xint";
     }
+    if (!debug_cmd.empty()) {
+        // enable debugger
+        options[vm_args.nOptions++].optionString = (char*)debug_cmd.c_str();
+    }
     if (!min_heap.empty()) {
         // set minimum heap size
         options[vm_args.nOptions++].optionString = (char*)min_heap.c_str();
@@ -88,6 +126,12 @@ QoreStringNode* Jvm::createVM() {
         // set maximum heap size
         options[vm_args.nOptions++].optionString = (char*)max_heap.c_str();
     }
+    if (!strvec.empty()) {
+        for (auto& str: strvec) {
+            //printd(5, "adding JVM pption: '%s'\n", str.c_str());
+            options[vm_args.nOptions++].optionString = (char*)str.c_str();
+        }
+    }
 
 #ifdef QORE_JNI_SUPPORT_CLASSPATH
     if (!classpath.empty()) {
@@ -103,6 +147,10 @@ QoreStringNode* Jvm::createVM() {
     if (rc != JNI_OK) {
         return new QoreStringNodeMaker("JNI_CreateJavaVM() failed with error code %d", rc);
     }
+    // now the JVM will also enforce the thread size on the primary thread
+#ifdef _QORE_HAS_ENFORCE_THREAD_SIZE_ON_PRIMARY_THREAD
+    q_enforce_thread_size_on_primary_thread();
+#endif
     return 0;
 }
 
